unit Memogam;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs;

Const
 MaxX=6;
 MaxY=4;
 ImgW=70;
 ImgH=80;

type
  TBoard=array[0..MaxX-1,0..MaxY-1] of integer;

  TMemoryGame = class(TCustomControl)
  private
   fBoard:TBoard;
   fFlags:TBoard;
   fPictures:TBitmap;
   fPictureCount:integer;
   fFirstCard:integer;
   fFirstX,fFirstY:integer;
   fSecondCard:integer;
   fSecondX,fSecondY:integer;
   fCardsLeft:integer;
   EOnNewGame:TNotifyEvent;
   EOnBadMove:TNotifyEvent;
   EOnGoodMove:TNotifyEvent;
   EOnEndOfGame:TNotifyEvent;
   procedure SetPictures(Value:TBitmap);
   procedure SetPictureCount(Value:integer);
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  protected
   procedure SetCard(Index:integer);
   procedure Paint; override;
   procedure SetFlag(x,y,value:integer);
  public
   constructor Create(AOwner:TComponent); override;
   destructor Destroy; override;
   procedure Loaded; override;
   procedure NewGame;
  published
   Property Pictures:TBitmap read fPictures write SetPictures;
   Property PictureCount:integer read fPictureCount write SetPictureCount;
   Property OnNewGame:TNotifyEvent read EOnNewGame write EOnNewGame;
   Property OnBadMove:TNotifyEvent read EOnBadMove write EOnBadMove;
   Property OnGoodMove:TNotifyEvent read EOnGoodMove write EOnGoodMove;
   Property OnEndOfGame:TNotifyEvent read EOnEndOfGame write EOnEndOfGame;
   Property Color;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('MySoft.Fun', [TMemoryGame]);
end;

Constructor TMemoryGame.Create(AOwner:TComponent);
 begin
  inherited Create(AOwner);
  fPictures:=TBitmap.Create;
  Randomize;
 end;

Destructor TMemoryGame.Destroy;
 begin
  fPictures.Free;
  inherited Destroy;
 end;

Procedure TMemoryGame.Loaded;
 begin
  inherited Loaded;
  NewGame;
 end;

Procedure TMemoryGame.SetPictures(Value:TBitmap);
 begin
  fPictures.Assign(Value);
  invalidate;
 end;

Procedure TMemoryGame.SetPictureCount(Value:integer);
 begin
  fPictureCount:=Value;
  invalidate;
 end;

procedure TMemoryGame.SetCard(Index:integer);
 var
  x,y:integer;
 begin
  repeat
   x:=Random(MaxX);
   y:=Random(MaxY);
  until fBoard[x,y]=0;
  fBoard[x,y]:=Index;
 end;

Procedure TMemoryGame.NewGame;
 var
  i:integer;
 begin
  FillChar(fBoard,SizeOf(fBoard),0);
  FillChar(fFlags,SizeOf(fFlags),0);
  fCardsLeft:=0;
  for i:=1 to fPictureCount do begin
   SetCard(i);
   SetCard(i);
   inc(fCardsLeft);
  end;
  fFirstCard:=-1;
  fSecondCard:=-1;
  Invalidate;
  if Assigned(EOnNewGame) then EOnNewGame(Self);
 end;

Procedure TMemoryGame.Paint;
 var
  dc:HDC;
  mx,my:integer;
  fg:integer;
 begin
  dc:=CreateCompatibleDC(Canvas.Handle);
  SelectObject(dc,fPictures.Handle);
  for mx:=0 to maxx-1 do
   for my:=0 to maxy-1 do begin
    fg:=fFlags[mx,my];
    if fg>=0 then BitBlt(Canvas.Handle,ImgW*mx,ImgH*my,ImgW,ImgH,DC,ImgW*fg,0,SRCCOPY);
   end;
  DeleteDC(dc);
 end;

procedure TMemoryGame.SetFlag(x,y,value:integer);
 var
  r:TRect;
 begin
  if fFlags[x,y]=Value then exit;
  fFlags[x,y]:=Value;
  r:=Rect(ImgW*x,ImgH*y,ImgW*(x+1),ImgH*(y+1));
  InvalidateRect(Handle,@r,Value<0);
 end;

procedure TMemoryGame.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
 var
  cx,cy,fg:integer;
 begin
  if fCardsLeft=0 then begin
   NewGame;
   exit;
  end;
  cx:=X div ImgW;
  cy:=Y div ImgH;
  if fFirstCard=-1 then begin
   fFirstCard:=fBoard[cx,cy];
   fFirstX:=cx;
   fFirstY:=cy;
   SetFlag(fFirstX,fFirstY,fFirstCard);
  end else begin
   if (cx=fFirstX)and(cy=fFirstY) then exit;
   fSecondCard:=fBoard[cx,cy];
   fSecondX:=cx;
   fSecondY:=cy;
   SetFlag(fSecondX,fSecondY,fSecondCard);
  end;
 end;

procedure TMemoryGame.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
 var
  cx,cy:integer;
 begin
  cx:=X div ImgW;
  cy:=Y div ImgH;
  if fSecondCard<0 then exit;
  if fFirstCard=fSecondCard then begin
   if Assigned(EOnGoodMove) then EOnGoodMove(Self);
   fBoard[fFirstX,fFirstY]:=-1;
   fBoard[fSecondX,fSecondY]:=-1;
   SetFlag(fFirstX,fFirstY,-1);
   SetFlag(fSecondX,fSecondY,-1);
   dec(fCardsLeft);
   if fCardsLeft=0 then if Assigned(EOnEndOfGame) then EOnEndOfGame(Self);
  end else begin
   if Assigned(EOnBadMove) then EOnBadMove(Self);
   SetFlag(fFirstX,fFirstY,0);
   SetFlag(fSecondX,fSecondY,0);
  end;
  fFirstCard:=-1;
  fSecondCard:=-1;
 end;

end.
